@@ -26,7 +26,99 @@ my @types = (
   [ "DOUBLE",    "d1", 8],  # XXX 8-byte IEEE format
 );
 
-my %tags = (
+my %exif_intr_tags = (
+    0x1    => "InteroperabilityIndex",
+    0x2    => "InteroperabilityVersion",
+    0x1000 => "RelatedImageFileFormat",
+    0x1001 => "RelatedImageWidth",
+    0x1002 => "RelatedImageLength",
+);
+
+my %exif_tags = (
+    0x828D => "CFARepeatPatternDim",
+    0x828E => "CFAPattern",
+    0x828F => "BatteryLevel",
+    0x8298 => "Copyright",
+    0x829A => "ExposureTime",
+    0x829D => "FNumber",
+    0x83BB => "IPTC/NAA",
+    0x8769 => "ExifOffset",
+    0x8773 => "InterColorProfile",
+    0x8822 => { __TAG__ => "ExposureProgram",
+		1 => "Manual",
+		2 => "Program",
+		3 => "Aperture priority",
+		4 => "Shutter priority",
+		5 => "Program creative",
+		6 => "Program action",
+		7 => "Portrait",
+		8 => "Landscape",
+	      },
+    0x8824 => "SpectralSensitivity",
+    0x8825 => "GPSInfo",
+    0x8827 => "ISOSpeedRatings",
+    0x8828 => "OECF",
+    0x9000 => "ExifVersion",
+    0x9003 => "DateTimeOriginal",
+    0x9004 => "DateTimeDigitized",
+    0x9101 => "ComponentsConfiguration",
+    0x9102 => "CompressedBitsPerPixel",
+    0x9201 => "ShutterSpeedValue",
+    0x9202 => "ApertureValue",
+    0x9203 => "BrightnessValue",
+    0x9204 => "ExposureBiasValue",
+    0x9205 => "MaxApertureValue",
+    0x9206 => "SubjectDistance",
+    0x9207 => { __TAG__ => "MeteringMode",
+		1 => "Average",
+		2 => "Center weighted average",
+		3 => "Spot",
+		4 => "Multi-spot",
+		5 => "Multi-segment",
+	      },
+    0x9208 => { __TAG__ => "LightSource",
+		0 => "Auto",
+		1 => "Daylight",
+		2 => "Fluorescent",
+		3 => "Tungsten",
+		10 => "Flash",
+	      },
+    0x9209 => { __TAG__ => "Flash",
+		0 => "No",
+		1 => "Yes",
+	      },
+    0x920A => "FocalLength",
+    0x927C => "MakerNote",
+    0x9286 => "UserComment",
+    0x9290 => "SubSecTime",
+    0x9291 => "SubSecTimeOriginal",
+    0x9292 => "SubSecTimeDigitized",
+    0xA000 => "FlashPixVersion",
+    0xA001 => "ColorSpace",
+    0xA002 => "ExifImageWidth",
+    0xA003 => "ExifImageLength",
+    0xA005 => {__TAG__ => "InteroperabilityOffset",
+	       __SUBIFD__ => \%exif_intr_tags,
+	      },
+    0xA20B => "FlashEnergy",                  # 0x920B in TIFF/EP
+    0xA20C => "SpatialFrequencyResponse",     # 0x920C    -  -
+    0xA20E => "FocalPlaneXResolution",        # 0x920E    -  -
+    0xA20F => "FocalPlaneYResolution",        # 0x920F    -  -
+    0xA210 => { __TAG__ => "FocalPlaneResolutionUnit",     # 0x9210    -  -
+		1 => "dpi",
+		2 => "dpm",
+		3 => "dpcm",
+		4 => "dpmm",
+		5 => "dpm",
+	      },
+    0xA214 => "SubjectLocation",              # 0x9214    -  -
+    0xA215 => "ExposureIndex",                # 0x9215    -  -
+    0xA217 => "SensingMethod",                # 0x9217    -  -
+    0xA300 => "FileSource",
+    0xA301 => "SceneType",
+);
+
+my %tiff_tags = (
   255   => { __TAG__ => "SubfileType",
 	     1 => "FullResolution",
 	     2 => "ReducedResolution",
@@ -81,6 +173,9 @@ my %tags = (
   514   => "JPEGInterchangeFormatLngth",
   531   => "YCbCrPositioning",
   33432 => "Copyright",
+  34665 => { __TAG__ => "ExifOffset",
+	     __SUBIFD__ => \%exif_tags,
+	   },
 );
 
 
@@ -147,17 +242,18 @@ sub ifd
 
 sub tagname
 {
-    $tags{$_[1]} || "Tag-$_[1]";
+    $tiff_tags{$_[1]} || "Tag-$_[1]";
 }
 
 sub add_fields
 {
-    my($self, $offset, $ifds, $override_tags) = @_;
+    my($self, $offset, $ifds, $tags) = @_;
     return unless $offset;
-    $override_tags ||= {};
+    $tags ||= \%tiff_tags;
 
-    for (${$self->{source}}) {
+    for (${$self->{source}}) {  # alias as $_
 	my $entries = $self->unpack("x$offset n", $_);
+      FIELD:
 	for my $i (0 .. $entries-1) {
 	    my($tag, $type, $count, $voff) =
 		$self->unpack("nnNN", substr($_, 2 + $offset + $i*12, 12));
@@ -180,10 +276,14 @@ sub add_fields
 		}
 
 	    }
-	    $tag = $override_tags->{$tag} || $self->tagname($tag);
+	    $tag = $tags->{$tag} || $self->tagname($tag);
 
 	    if (ref($tag)) {
 		die "Assert" unless ref($tag) eq "HASH";
+		if (my $sub = $tag->{__SUBIFD__}) {
+		    $self->add_fields($val, $ifds, $sub);
+		    next FIELD;
+		}
 		$val = $tag->{$val} if exists $tag->{$val};
 		$tag = $tag->{__TAG__};
 	    }
